@@ -59,8 +59,22 @@ shortrq.fit.fnb <- function (x, y, tau = 0.5, beta = 0.99995, eps = 1e-06)
 }
 
 
-
-
+#' Implements QICD algorithm
+#' @param y response variable, length n vector
+#' @param x input nxp matrix, of dimension nobs x nvars; each row is an observation vector.
+#' @param tau the quantile value
+#' @param lambda the tuning parameter (numeric value > 0)
+#' @param intercept a logical value,should intercept be fitted (default=TRUE) (intercept should be included when using splines)
+#' @param penalty The name of the penalty function ("SCAD", "MCP", "LASSO")
+#' @param initial_beta Vector containing initial values for intercept (if included) and x coefficients. Should be in the form (intercept, coefficients) intercept should be left out if intercept=FALSE. 
+#' @param maxin maximum number of iterations for inside coordinate descent,default value is 100
+#' @param maxout maximum number of iterations for outside MM step, default value is 20
+#' @param eps The convergence threshold for coordinate descent and majorization minimization step
+#' @param coef.cutoff Threshold for determining nonzero coefficients
+#' @param a Scale parameter, the default value is 3.7 (>2 for SCAD, >1 for MCP, not used in LASSO)
+#' @param scalex Whether predictors are centered and scaled
+#' @param ... additional parameters
+#' @export
 QICD <- function(y, x, tau=.5, lambda, intercept=TRUE, penalty="SCAD", 
                  initial_beta=NULL, maxin=100, maxout=20, eps = 1e-05, coef.cutoff=1e-08,  
                  a=3.7, scalex=TRUE, ...)
@@ -99,7 +113,7 @@ QICD <- function(y, x, tau=.5, lambda, intercept=TRUE, penalty="SCAD",
   if( is.null(initial_beta) ){
     # initial_beta <- LASSO.fit(y, x, tau, lambda, intercept, coef.cutoff)
     initial_beta <- coefficients( cv.rq.pen(x, y, tau=tau, intercept=intercept, 
-                            penalty="LASSO", criteria="BIC") )
+                            penalty="LASSO", criteria="BIC",internal=TRUE) )
   }
 
   if( intercept ){
@@ -170,7 +184,25 @@ QICD <- function(y, x, tau=.5, lambda, intercept=TRUE, penalty="SCAD",
 }
 
 
-
+#' Implements QICD algorithm with some variables not being penalized
+#' @param y response variable, length n vector
+#' @param x input nxp matrix, of dimension nobs x nvars; each row is an observation vector.
+#' @param z nxq matrix of bases; the coefficients for these columns will be unpenalized
+#' @param tau the quantile value
+#' @param lambda the tuning parameter (numeric value > 0)
+#' @param intercept a logical value,should intercept be fitted (default=TRUE) (intercept should be included when using splines)
+#' @param maxin maximum number of iterations for inside coordinate descent,default value is 100
+#' @param maxout maximum number of iterations for outside MM step, default value is 20
+#' @param eps The convergence threshold for coordinate descent and majorization minimization step
+#' @param penalty The name of the penalty function ("SCAD", "MCP", "LASSO")
+#' @param a Scale parameter, the default value is 3.7 (>2 for SCAD, >1 for MCP, not used in LASSO)
+#' @param coef.cutoff Threshold for determining nonzero coefficients
+#' @param initial_beta Vector containing initial values for intercept (if included) and x coefficients. Should be in the form (intercept, coefficients) intercept should be left out if intercept=FALSE. The intercept should be included to be consistent with other methods, but intercept and z coefficients will be initialized to by a rq() fit of residuals from initial beta against the unpenalized predictors, z.
+#' @param method quantile regression initialization method, can be "br" or "fn".
+#' @param scalex Whether predictors are centered and scaled
+#' @param ... additional parameters
+#' 
+#' @export
 QICD.nonpen <- function(y, x, z, tau=.5, lambda, intercept=TRUE, penalty="SCAD", 
                  initial_beta=NULL, maxin=100, maxout=20, eps = 1e-05, coef.cutoff=1e-08,  
                  a=3.7, method="br", scalex=TRUE, ...)
@@ -331,7 +363,7 @@ QICD.nonpen <- function(y, x, z, tau=.5, lambda, intercept=TRUE, penalty="SCAD",
 
 QICD.group <- function(y, x, groups, tau=.5, lambda, intercept=TRUE, penalty="SCAD", 
                  initial_beta=NULL, maxin=100, maxout=20, eps = 1e-05, coef.cutoff=1e-08,  
-                 a=3.7, scalex=TRUE, ...)
+                 a=3.7, scalex=TRUE, norm=2, ...)
 #y: response variable, length n vector
 #x: input nxp matrix, of dimension nobs x nvars; each row is an observation vector. 
 #groups: numeric vector of length ncol(x) with the group number of the coefficient (can be unique)
@@ -399,7 +431,7 @@ QICD.group <- function(y, x, groups, tau=.5, lambda, intercept=TRUE, penalty="SC
 
   i=0
   distance <- eps+1
-  groupl1 <- rep(0, p)
+  groupNorm <- rep(0, p)
   beta0 <- beta
   intval0 <- intval
   residuals <- as.double(y - x%*%beta - intval)
@@ -409,10 +441,14 @@ QICD.group <- function(y, x, groups, tau=.5, lambda, intercept=TRUE, penalty="SC
   while( (i < maxout) & (distance >= eps) ){
 
     for(grps in unique(groups)){
-      groupl1[groups==grps] <- sum( abs(beta[groups==grps]) )
+	  if(norm==2){
+		groupNorm[groups==grps] <- sqrt( sum( beta[groups==grps]^2) )
+	  } else{
+		groupNorm[groups==grps] <- sum( abs(beta[groups==grps]) )
+	  }
     }
 	#lambda <- n*lambda
-    out <- .C("penderiv", as.double(groupl1), p, a, lambda, pentype)
+    out <- .C("penderiv", as.double(groupNorm), p, a, lambda, pentype)
 	#penweight <- as.double(out[[1]])
     penweight <- as.double( n*out[[1]] )
 
